#!/usr/bin/perl
use strict;

#
# * Program: netmap.cgi
# * Version: 0.0.1 
# * License: GPL
# * Copyright (c) 1999 Ivan Kuncl (kuncl@vsbohem.cz)
# *
# * First Written:   08-26-1999 (start of development for version 0.0.1)
# * Last Modified:   08-29-1999
# *
# * Description:
# *
# * netmap.cgi draw a map of your net,
# * 
# * Parameters: map=name_of_map
# *
# * netmap.cgi use netmap.cfg, which is in the same directory as 
# * netsaint main configuration file, and several picture from
# * directory {/usr/local}/netsaint/share/images/

# Remark: All host must be in odd columns !!!

my $NSCGI="./nscgi.cfg"; # Main configuration file for cgi programs

my($STRUCTURE,$STATUS);

open(NSCGI,"$NSCGI") or die("I cannot open $NSCGI");
while(<NSCGI>) {
 /^main_config_file=(.*)\W/ && do { $STRUCTURE=$1;last; }
}
close(NSCGI);

# Status file
open(MAIN,"$STRUCTURE") or die("I cannot open $STRUCTURE");
while(<MAIN>) {
 /^status_file=(.*)\W/ && do { $STATUS=$1;last; }
}
close(MAIN);

# netmap.cfg is in the same directory as main config file

$STRUCTURE =~ s/(.*)\/(.*)$/$1\/netmap.cfg/;

my $PICTURE="<img border=\"0\" src=\"/netsaint/images/PICTURE\" height=\"30\" >";

my %HOST_PICTURE = ( "UP" => "/netsaint/images/host-up.gif",
                     "UP,WARN" => "/netsaint/images/host-up-warn.gif",
                     "UP,CRIT" => "/netsaint/images/host-up-crit.gif",
                     "UP,WARN,CRIT" => "/netsaint/images/host-up-warn-crit.gif",
                     "DOWN" => "/netsaint/images/host-down.gif",
                     "UNRECHEABLE" => "/netsaint/images/host-down.gif");
my %HOSTS;

# Name of map
my $MAP=$ENV{QUERY_STRING};

# For v.1.00 only one parameter: map=...
$MAP=~ s/^map=//;
$MAP="all" if $MAP eq "";

# At first, we read status of hosts

open(STATUS,"$STATUS") or die("I cannot open $STATUS");

while(<STATUS>) {
  /^\[.+\] HOST;(.+?);(.+?);.*/ && do { $HOSTS{$1}{state}=$2; };
  /^\[.+\] SERVICE;(.+?);.+?;(.+?);.*/ && do {
     ($HOSTS{$1}{warn}=defined($HOSTS{$1}{warn})?(1+$HOSTS{$1}{warn}):1) if $2 eq "WARNING";
     ($HOSTS{$1}{crit}=defined($HOSTS{$1}{crit})?(1+$HOSTS{$1}{crit}):1) if $2 eq "CRITICAL";
  }
}
close(STATUS);

# Next, we read map of the net

my (@line);

# Data structure
#
#  All is in one two-dimensional array
my (@MAP);
#   element of array is hash of 
#    'picture','state' and 'text'
open(STRUCTURE,"$STRUCTURE") or die("I cannot open $STRUCTURE");

my($line,$column,$bit,$mline,$mcolumn,$mapcnt,%NETS);
while(<STRUCTURE>) {
 /^map_property\[(.+)\]=([^;]+);?(.*)$/ && do { # Now, we count nets
    $NETS{$1}{full_name}=$2;$mapcnt++;
    $NETS{$1}{hosts}=$3;
    $NETS{$1}{warn}=0;
    $NETS{$1}{crit}=0;
    $NETS{$1}{down}=0;
 };
 /^map\[$MAP\]=(.*)/ && do {
    (@line)=split(/;/,$1);
    foreach $bit (@line) {
      chomp($bit);
      $bit =~ s/\s//g;
      if ( $bit =~ /^(l42|l24)$/ ) { # Horizontal line
        $MAP[$line][$column]{picture}="l42";
        $MAP[$line][$column]{state}=$MAP[$line][$column-1]{state};
        $MAP[$line][$column]{text}="";                
        $MAP[$line][$column]{picture}.=$MAP[$line][$column]{state} =~ /^(UP|PENDING)/ ? "-up.gif":"-down.gif"  ;
      }
      elsif ( $bit =~ /^(l432|l423|l342|l324|l234|l243)$/ ) { # Horizontal line and down
        $MAP[$line][$column]{picture}="l432";
        $MAP[$line][$column]{state}=$MAP[$line][$column-1]{state};
        $MAP[$line][$column]{text}="";                
        $MAP[$line][$column]{picture}.=$MAP[$line][$column]{state} =~ /^(UP|PENDING)/ ? "-up.gif":"-down.gif"  ;
      } elsif ( $bit =~ /^(l12|l21)$/ ) { # top-to-right
        $MAP[$line][$column]{picture}="l12";
        $MAP[$line][$column]{state}=$MAP[$line-1][$column]{state};
        $MAP[$line][$column]{text}="";              
        $MAP[$line][$column]{picture}.=$MAP[$line][$column]{state} =~ /^(UP|PENDING)/ ? "-up.gif":"-down.gif"  ;
      } elsif ( $bit =~ /^(l123|l132|l213|l231|l312|l321)$/ ) { # top to right and down
        $MAP[$line][$column]{picture}="l123";
        $MAP[$line][$column]{state}=$MAP[$line-1][$column]{state};
        $MAP[$line][$column]{text}="";        
        $MAP[$line][$column]{picture}.=$MAP[$line][$column]{state} =~ /^(UP|PENDING)/ ? "-up.gif":"-down.gif"  ;
      } elsif ( $bit =~ /^(l13|l31)$/ ) { # top to down
        $MAP[$line][$column]{picture}="l13";
        $MAP[$line][$column]{state}=$MAP[$line-1][$column]{state};
        $MAP[$line][$column]{text}="";        
        $MAP[$line][$column]{picture}.=$MAP[$line][$column]{state} =~ /^(UP|PENDING)/ ? "-up.gif":"-down.gif"  ;
      } elsif ( $bit eq "" ) { # Nothing is in this cell
        $MAP[$line][$column]{picture}="";
        $MAP[$line][$column]{state}="";
        $MAP[$line][$column]{text}="";        
      } else { # Host
        $MAP[$line][$column]{picture}="";
        $MAP[$line][$column]{state}=$HOSTS{$bit}{state};
        $MAP[$line][$column]{text}="$bit";        
# Dependence
        if ( $HOSTS{$bit}{state} =~ /^UP.*/ && 
            ( 
              ( $column > 0 && $MAP[$line][$column-1]{state} !~ /^(UP|PENDING).*/   && ($MAP[$line][$column-1]{picture}) =~ /l.*2.*/) ||
              ( $line > 0 && $MAP[$line-1][$column]{state} !~ /^(UP|PENDING).*/ && $MAP[$line-1][$column]{picture} =~ /l\d*3\d*/ ) 
            ) 
           ) {
            $MAP[$line][$column]{state}="DOWN";
        }

        # Warn and crit state
        $MAP[$line][$column]{state}.=",WARN" if ( $MAP[$line][$column]{state} =~ /^UP.*/ && $HOSTS{$bit}{warn} > 0 );
        $MAP[$line][$column]{state}.=",CRIT" if ( $MAP[$line][$column]{state} =~ /^UP.*/ && $HOSTS{$bit}{crit} > 0 );
      }
      $column++; 
    }
   $mcolumn = $column > $mcolumn ? $column:$mcolumn; 
   $line++;$column=0;
 }
}
close(STRUCTURE);
$mline=$line;

# Creation of html page

print "Content-Type: text/html\n\n";
print "<html>";
print "<head><META HTTP-EQUIV=\"Refresh\" CONTENT=\"120\">
             <META HTTP-EQUIV=\"Pragma\" CONTENT=\"no-cache\">
       <title>
	     Network map
        </title>
       </head>";

# Subnet table

my ($n,$h);
foreach $n ( keys(%NETS) ) {
  foreach $h ( split(/,/,$NETS{$n}{hosts} )) {
    $NETS{$n}{warn}+=$HOSTS{$h}{warn};
    $NETS{$n}{crit}+=$HOSTS{$h}{crit};    
    $NETS{$n}{down}+=1 if $HOSTS{$h}{state} eq "DOWN";
  } 
} 

print "<body>\n";
# Subnet table
if ( $mapcnt > 1 ) {
 print "<font size=\"2\"><table BORDER=\"1\" bgcolor=\"#EEEEEE\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\"><tr><td rowspan=\"3\">Subnets</td>";
 foreach ( keys(%NETS) ) { 
   print "<td colspan=3><a href=\"/cgi-bin/netsaint/netmap.cgi?map=$_\">$NETS{$_}{full_name}</a></td>"; 
 }
print "</tr><tr>";

 foreach ( keys(%NETS) ) { 
   print "<td>Down</td><td>Warn</td><td>Crit</td>"; 
 }

print "</tr><tr>";

 foreach ( keys(%NETS) ) { 
   print "<td",($NETS{$_}{down}>0 ? " bgcolor=\"#F83838\" ":" ") , 
         ">$NETS{$_}{down}</td>",
	 "<td",($NETS{$_}{warn}>0 ? " bgcolor=\"#FFFF00\" ":" ") ,">$NETS{$_}{warn}</td><td",
	 ($NETS{$_}{crit}>0 ? " bgcolor=\"#F83838\" ":" ") ,">$NETS{$_}{crit}</td>"; 
 }

 
 print "</tr></table></font>\n";
}

print "<p>&nbsp<div align=\"center\"><table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">";

# End of subnet table

my ($temp,$cell,$celltype);
for $line ( 0 .. $mline) {

# Line begin
    print "<tr>\n";

   for $column (0 .. $mcolumn) {
# Print cell
     $celltype=( ($column/2) == int($column/2) ) ? 1:0;
     $cell=$PICTURE;
     $temp=$MAP[$line][$column]{picture};

     if ( $temp ne "" ) {  # line cell
      $temp="small-" . $temp if ! $celltype;
      $cell =~ s/PICTURE/$temp/g;
      print "<th width=\"",$celltype?20:20,"\" height=\"30\">",$cell,"</th>\n";
     }  # $MAP[$line][$column]{text}
     else {   # host cell
      if ( $MAP[$line][$column]{text} eq "" ) { # Nothing
       print "<th></th>\n";
      } else {
       print "<th width=\"60\" nowrap height=\"30\" background=\"$HOST_PICTURE{$MAP[$line][$column]{state}}\">";
       print "<font size=\"2\">";
       print "<a onmousemove=\"window.status=&quot;Server: $MAP[$line][$column]{text}";
       print ",$HOSTS{$MAP[$line][$column]{text}}{warn} warning" if $HOSTS{$MAP[$line][$column]{text}}{warn} > 0;
       print ",$HOSTS{$MAP[$line][$column]{text}}{crit} critical"  if $HOSTS{$MAP[$line][$column]{text}}{crit} > 0;
       print "&quot;\" href=\"/cgi-bin/netsaint/status.cgi?host=$MAP[$line][$column]{text}\" >";
       print ucfirst(substr($MAP[$line][$column]{text},0,10));
       print "</a>";
       print "</font>";
       print "</th>\n";
      }
     }
   }
   print "</tr>\n";
}

print "</table></body></html>\n";

